/////////////////////////////////////////////////////////////
//	A/D Module Source Code of AutoDcReg.
/////////////////////////////////////////////////////////////
#include	<p18f1320.h>
#include	<string.h>
#include	<delays.h>
#include	"AutoDcReg.h"
#include	"Prototype.h"


static	int			m_BatBuf[10];
static	int			m_LipBuf[10];
static	char		m_LipOffCt;
static	char		m_LipOnCt;
static	char		m_V50BatOffCt;
static	char		m_V50BatOnCt;
static	char		m_V25BatOffCt;
static	char		m_V25BatOnCt;


//////////////////////////////////////
//	A/D Module Initialize
/////////////////////////////////////
void	adInit()
{
	// A/D PORT Setting
	ADCON0 = 0b00000001; 
	ADCON1 = 0b01111100;		// A/D ON
	ADCON2 = 0b10101101;		// Conversion Clock is 16*Tosc
								// A/D Acquisition Time is 20*TAD
 	memset((void*)&m_BatBuf,0,sizeof(m_BatBuf));
	adLippleStatusClr();
	adBatV25StatusClr();
	adBatV50StatusClr();

}

///////////////////////////////////////////////////
//	Battry A/D Value Get routine, called by 1sec
//	<out> Battery A/D Value(avarage)
//////////////////////////////////////////////////
static	char	lc_i;
static	char	lc_j;
//static	int		lc_d;
static	int		lc_sum;

int	adGetBattery()
{
	lc_sum = 0;
	for(lc_i=0;lc_i<10;lc_i++){
//		lc_d = m_BatBuf[lc_i];
		lc_sum += m_BatBuf[lc_i];
	}
	return(lc_sum/10);
}


///////////////////////////////////////////////////
//	Battry Voltage A/D Converter Exec
//	<out> m_BatBuf[]
//////////////////////////////////////////////////
static	char	lc_buf_ix=0;
void	adCnvBattery()
{
	// Battery A/D Start
	ADCON0bits.CHS0 = 0;
	Delay10TCYx(5);
	ADCON0bits.GO = 1; 
	Delay10TCYx(20);
	m_BatBuf[lc_buf_ix] = (ADRES & 0b000001111111111);
	lc_buf_ix ++;
	if (lc_buf_ix >=10)lc_buf_ix = 0;
}

int adCnvBattery1()
{
	ADCON0bits.CHS0 = 0;
	Delay10TCYx(5);
	ADCON0bits.GO = 1; 
	Delay10TCYx(20);
	return((ADRES & 0b000001111111111));
}

///////////////////////////////////////////////////
//	Lipple A/D Value Get routine, called by 1sec
//	<out> Lipple A/D Value(avarage)
//////////////////////////////////////////////////
#define	N	10
int 	lc_avg;
int 	lc_temp;
int	adGetLipple()
{
	BSort(m_LipBuf,N);
	lc_avg = 0;
	for(lc_i=4;lc_i<8;lc_i++){
		lc_avg += m_LipBuf[lc_i];
	}
	return(lc_avg/4);
}
///////////////////////////////////////////////////
//	Battry Lipple A/D Converter Exec
//	<out> m_LipBuf[]
//////////////////////////////////////////////////
int	ad;
int	i;

static	char	buf_ix=0;
int max_ad,min_ad;
void	adCnvLipple()
{

	max_ad = 0;
	min_ad = 1024;

	// Lipple A/D Start
	ADCON0bits.CHS0 = 1;
	Delay10TCYx(5);
	for(i=0;i<800;i++){
		ADCON0bits.GO = 1; 
		while(ADCON0bits.GO != 0); 
		ad = ADRES & 0b000001111111111;
		if (ad > max_ad)max_ad=ad;
		if (ad < min_ad)min_ad=ad;
	}
	m_LipBuf[buf_ix] = max_ad - min_ad;
	buf_ix ++;
	if (buf_ix>=10)buf_ix = 0;

}


////////////////////////////////////////////////////////////////
//	Lipple Status Clear
///////////////////////////////////////////////////////////////
void	adLippleStatusClr()
{
	m_LipOffCt 	= TM_LIPPLE_OFF-1;
	m_LipOnCt	= TM_LIPPLE_ON-1;
}
////////////////////////////////////////////////////////////////
//	Lipple Status Get call by 1 sec interval
//
//	<in> 	ad	 :Lipple A/D value
//	     	th_lipple_ad :Threshold Lipple ON A/D value
//	<out> 	LIPPLE_ON :10sec Lipple is detected.
//		 	LIPPLE_OFF:3sec Lipple is detected.
//	      	LIPPLE_INVALID:Lipple Invalid
///////////////////////////////////////////////////////////////
char	adGetLippleStatus(int ad,int th_lipple_ad)
{
	if (ad < th_lipple_ad){
		m_LipOnCt	= TM_LIPPLE_ON-1;
		if (m_LipOffCt != 0){
			m_LipOffCt --;
			return(LIPPLE_INVALID);		
		}else{
			return(LIPPLE_OFF);		
		}
	}else{
		m_LipOffCt 	= TM_LIPPLE_OFF-1;
		if (m_LipOnCt != 0){
			m_LipOnCt --;
			return(LIPPLE_INVALID);		
		}else{
			return(LIPPLE_ON);		
		}

	}

}

////////////////////////////////////////////////////////////////
//	Battry Capacity 25% 60sec Check
//
//	<in> 	ad	 	:Battery A/D value
//			th_v25	:Battery Capacity 25% ThreshHold
//	<out> 	BAT_V25_ON :30 sec V25% Over is detected.
//	      	BAT_V25_OFF:10 sec V25% Under is detected
//	      	BAT_V25_INVALID:V25% Invalid
///////////////////////////////////////////////////////////////
char	adGetBatV25Status(int ad,int th_v25)
{
	if (ad < th_v25){
		m_V25BatOnCt	= TM_BAT_V25_ON-1;
		if (m_V25BatOffCt != 0){
			m_V25BatOffCt --;
			return(BAT_V25_INVALID);		
		}else{
			return(BAT_V25_UNDER);		
		}
	}else{
		m_V25BatOffCt 	= TM_BAT_V25_OFF-1;
		if (m_V25BatOnCt != 0){
			m_V25BatOnCt --;
			return(BAT_V25_INVALID);		
		}else{
			return(BAT_V25_OVER);		
		}
	}
}
////////////////////////////////////////////////////////////////
//	Battry Capacity 25% 60sec Check status clear
///////////////////////////////////////////////////////////////
void	adBatV25StatusClr()
{
	m_V25BatOffCt 	= TM_BAT_V25_OFF-1;
	m_V25BatOnCt	= TM_BAT_V25_ON-1;
}

////////////////////////////////////////////////////////////////
//	Battry Capacity 50% 60sec Check
//
//	<in> 	ad	 	:Battery A/D value
//			th_v50	:Battery Capacity 50% ThreshHold
//	<out> 	BAT_V50_ON :30 sec V50% Over is detected.
//	      	BAT_V50_OFF:30 sec V50% Under is detected
//	      	BAT_V50_INVALID:V50% Invalid
///////////////////////////////////////////////////////////////
char	adGetBatV50Status(int ad,int th_v50)
{

	if (ad < th_v50){
		m_V50BatOnCt	= TM_BAT_V50_ON-1;
		if (m_V50BatOffCt != 0){
			m_V50BatOffCt --;
			return(BAT_V50_INVALID);		
		}else{
			return(BAT_V50_UNDER);		
		}
	}else{
		m_V50BatOffCt 	= TM_BAT_V50_OFF-1;
		if (m_V50BatOnCt != 0){
			m_V50BatOnCt --;
			return(BAT_V50_INVALID);		
		}else{
			return(BAT_V50_OVER);		
		}
	}
}

////////////////////////////////////////////////////////////////
//	Battry Capacity 50% 60sec Check status clear
///////////////////////////////////////////////////////////////
void	adBatV50StatusClr()
{
	m_V50BatOffCt 	= TM_BAT_V50_OFF-1;
	m_V50BatOnCt	= TM_BAT_V50_ON-1;
}

