//////////////////////////////////////////////////////////////////////////////////////////
//	 Auto Power On/Off DC Regurator(Relay)
//
//	PIC18F1320
//
//
//////////////////////////////////////////////////////////////////////////////////////////
#include	<p18f1320.h>
#include	<stdio.h>
#include	<reset.h>
#include	<string.h>
#include	<delays.h>


#include	"CpuOption.h"
#include	"AutoDcReg.h"
#include	"Prototype.h"
#include	"Io.h"
#include	"Global.h"

// Local function
void 	Goto_PowerUpState(T_ROM msg);
void 	Goto_StanbyState(T_ROM msg);
void 	Goto_StartUpState(T_ROM msg);
void	Goto_OutputState(T_ROM msg);
void	Goto_BatWarningState(T_ROM msg);
void	Goto_BatLowState(T_ROM msg);
void	Goto_BreakDownState(T_ROM msg);
void	BatteryLongevityExec(T_ROM msg);
void 	sleep(char n);

/////////////////////////////////////////////////////////////////////
//	Main Routine
/////////////////////////////////////////////////////////////////////
int BatAd;					// Battery A/D Average
int LipAd;					// Lipple A/D Average
int BatAd1;					// Battery A/D
char startup_led;			// Led Flag
char sleep_ct;				// Sleep Counter;
char LvStatus;				// Lipple Status
char Bv25Status;			// Battery Capacity 25% Status
char Bv50Status;			// Battery Capacity 50% Status
char PTimer;				// Power On Timer;
void	main()
{
T_BAT_TH_V	BatThV;			// Battery Check Level
int Bv;						// Battry Voltage Average
int Lv;						// Lipple Voltage Average
int Bv1;					// Battry Voltage
int Bv1Bak;					// Battry Voltage
char BatVlFg;

	// OSC SETTING 8MHz,PRI_RUN mode
	OSCCON=0x70;
	// A/D Initialize
	adInit();
	// I/O PORT INITIALIZE
	ioInitPortRa();
	ioInitPortRb();
	ioFlash();
	// Uart Init
	uartInit();
	// Display Start
	uartPutStrValue((T_ROM) "\033[2J",NON_VALUE);	// clr
	uartPutStrValue((T_ROM) VERSION,NON_VALUE);	// Version
	// Start PowerUp State
	PTimer = 0;
	Goto_PowerUpState((T_ROM)"--->Power Up");
	// Calibration Data Read
	calDataRead(&BatThV);

	sleep_ct=0;							// Sleep Counter;
	//////////////////////
	// Main Loop
	//////////////////////
	while(1){
		// Sleep Wait Timer
		if (gMainState == ST_STANBY){
			sleep(SL_STANBY);		// 1.024s
	        sleep_ct=SL_STANBY;
		}else{
			sleep(SL_OTHERS);		// 64ms
			// Lipple Ad Convert
			adCnvLipple();
			sleep_ct++;
		}
		if (sleep_ct >= SL_1S){
			sleep_ct = 0;
			//////////////////////
			// 1 sec interval
			//////////////////////
			// Battry Ad 1 Convert
			BatAd1 	= adCnvBattery1();
			Bv1 =	AdToV(BatAd1);
			// Battry Ad Convert
			adCnvBattery();
			// A/D Read
			BatAd 	= adGetBattery();
			LipAd  	= adGetLipple();
			// Volt Convert
			Bv =	AdToV(BatAd);
			Lv  =	AdToV5(LipAd);
			// Lipple Status Check
			LvStatus =adGetLippleStatus(Lv,BatThV.LvTh);
			// Battery Capacity 25% Status check
			Bv25Status = adGetBatV25Status(Bv,BatThV.V25);
			// Battery Capacity 50% Status check
			Bv50Status = adGetBatV50Status(Bv,BatThV.V50);

			////////////////////////////
			// Lipple Led Output
			////////////////////////////
			if (LvStatus == LIPPLE_OFF || gMainState == ST_BAT_LOW || gMainState == ST_STANBY){
				PORT_LLED = LLED_OFF;
			}else if (LvStatus == LIPPLE_ON){
				PORT_LLED = LLED_ON;
			}
			////////////////////////////
			// Debug Data Output
			////////////////////////////
			uartPutStrValue((T_ROM) "\nState=",gMainState);
			uartPutStrValue((T_ROM) "Bv=",Bv);
			uartPutStrValue((T_ROM) "Lv=",Lv);
			uartPutStrValue((T_ROM) "LvStatus=",LvStatus);
			uartPutStrValue((T_ROM) "Bv50Status=",Bv50Status);
			uartPutStrValue((T_ROM) "Bv25Status=",Bv25Status);
			uartPutStrValue((T_ROM) "Bv1=",Bv1);


			BatAd1 	= adCnvBattery1();
			Bv1Bak =	AdToV(BatAd1);
			if (Bv1 <= BatThV.Vl && Bv1Bak <= BatThV.Vl){
				BatVlFg = 1;
			}else{
				BatVlFg = 0;
			}
			////////////////////////////
			// Main State Machine.
			////////////////////////////
			switch(gMainState){
			case ST_POWER_UP:
				/////////////////////////
				// Power Up State.
				/////////////////////////
				if (PTimer >= SL_10S){
					// Calibrtion Exec when DSW Assert
					calExec(&BatThV);
					Goto_StanbyState((T_ROM)"--->Stanby(After PowerUp)");
				}else{
					PTimer ++;
				}
				break;
			case ST_STANBY:
				/////////////////////////
				// STANBY State.
				/////////////////////////
				if (BatVlFg@!=0 ){
					// In case of Battery Longevity
					BatteryLongevityExec((T_ROM) "--->Under Vl");
				}else if(Bv25Status == BAT_V25_UNDER){
					Goto_BatLowState((T_ROM)"--->BatLow(Under V25)");
				}else if (Bv >= BatThV.Vc){
					startup_led = LED_OFF;
					Goto_StartUpState((T_ROM) "--->StartUp(Over Vc)");
				}
				break;
			case ST_START_UP:
				/////////////////////////
				// Start Up State.
				/////////////////////////
				if (Lv >= BatThV.LvTh){
					if (startup_led == LED_OFF){
						startup_led = LED_ON;
						LedSet(LED_START_UP);
					}
				}
				if (Bv < BatThV.V25){
					Goto_StanbyState((T_ROM)"--->Stanby(Under V50)");
				}else if(LvStatus == LIPPLE_OFF){
					Goto_StanbyState((T_ROM)"--->Stanby(Lipple Off)");
				}else if (Bv >= BatThV.V50 && LvStatus == LIPPLE_ON){
					Goto_OutputState((T_ROM) "-->Output State(Over V50 & Lipple On)");
				}
				if (Bv >= BatThV.Vbd){
					Goto_BreakDownState((T_ROM) "-->BreakDown(Over Vbd)");
				}
				break;
			case ST_OUTPUT:
				/////////////////////////
				// Output State.
				/////////////////////////
				if (Bv >= BatThV.Vbd){
					Goto_BreakDownState((T_ROM) "-->BreakDown(Over Vbd & Lipple Off)");
				}else if(LvStatus == LIPPLE_OFF){
					Goto_StanbyState((T_ROM)"--->Stanby(Lipple Off)");
				}else if(Bv50Status == BAT_V50_UNDER){
					Goto_BatWarningState((T_ROM)"--->BatWarning(Under V50)");
				}else if(Bv25Status == BAT_V25_UNDER){
					Goto_BatLowState((T_ROM)"--->Bat Low(Under V25)");
				}else{

				}
				break;
			case ST_BAT_WARNING:
				/////////////////////////////
				// Battery Warning State.
				/////////////////////////////
				if (Bv50Status == BAT_V50_OVER){
					Goto_OutputState((T_ROM) "-->Output State(Over V50)");
				}else if(LvStatus == LIPPLE_OFF){
					Goto_StanbyState((T_ROM)"--->Stanby(Lipple Off)");
				}else if(Bv25Status == BAT_V25_UNDER){
					Goto_BatLowState((T_ROM)"--->Bat Low(Under V25)");
				}
				break;
			case ST_BAT_LOW:
				if(Bv25Status == BAT_V25_OVER){
					Goto_StanbyState((T_ROM)"--->Stanby(Over V25)");
				}
				if (BatVlFg@!=0){
					BatteryLongevityExec((T_ROM) "--->Under Vl");
					Goto_StanbyState((T_ROM)"--->Stanby(Vl End)");
				}
				break;
			case ST_BREAK_DOWN:
				if(Bv <= (BatThV.VbdTh)){
					Goto_StanbyState((T_ROM)"--->Stanby(Under Vbd & Lipple On)");
				}
				break;
			default:
				break;
			}
		}
		//////////////////////
		// 120ms interval
		//////////////////////
		// Wdt
		ClrWdt();
		// Charge Led Control
		LedControl();
		// I/O Flash
		ioFlash();
	}

}
// Change PowerUp State
void Goto_PowerUpState(T_ROM msg)
{
	LedSet( LED_POWER_UP_STATE );
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	PORT_RY = OFF;
	gMainState = ST_POWER_UP;
}
// Change Stanby State
void Goto_StanbyState(T_ROM msg)
{
	LedSet(LED_STANBY_STATE);
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	PORT_RY = OFF;
	gMainState = ST_STANBY;
	LedControl();
	ioFlash();
	sleep(3000/64);		// 3s
}
// Change StartUp State
void Goto_StartUpState(T_ROM msg)
{
	adLippleStatusClr();
	adBatV25StatusClr();
	adBatV50StatusClr();
	uartPutStrValue(msg,NON_VALUE);
	PORT_RY = OFF;
	gMainState = ST_START_UP;

}
// Change Output State
void	Goto_OutputState(T_ROM msg)
{
	LedSet(LED_OUTPUT_STATE);
	uartPutStrValue(msg,NON_VALUE);
	PORT_RY = ON;
	gMainState = ST_OUTPUT;
}
// Change Bat Warning State
void	Goto_BatWarningState(T_ROM msg)
{
	LedSet(LED_BAT_WARNING);
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	PORT_RY = ON;
	gMainState = ST_BAT_WARNING;
}
// Change Bat Low State
void	Goto_BatLowState(T_ROM msg)
{
	LedSet(LED_BAT_LOW_STATE);
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	PORT_RY = OFF;
	gMainState = ST_BAT_LOW;
}
// Change Break Down State
void	Goto_BreakDownState(T_ROM msg)
{
	LedSet(LED_BREAK_DOWN_STATE);
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	PORT_RY = OFF;
	gMainState = ST_BREAK_DOWN;
}
// BatteryLongevity State
char led_time;
void	BatteryLongevityExec(T_ROM msg)
{
	uartPutStrValue((T_ROM) msg,NON_VALUE);
	for(led_time=0;led_time<VL_LED_TIME;led_time ++){
		PORT_DIRECT_MLED=MLED_ON; 
		ClrWdt();
		Delay1KTCYx(DLY_LED_ON_BAT_LONGEVITY_TIME); // 50ms Wait
		ClrWdt();
		PORT_DIRECT_MLED=MLED_OFF; 
		Delay1KTCYx(DLY_LED_OFF_BAT_LONGEVITY_TIME); // 50ms Wait
		ClrWdt();
	}
}
void sleep(char n)
{
char i;
	for(i=0;i<n;i++)Sleep();
}