//////////////////////////////////////////////////////////////////////////////////////////
// 
//	Calibration Modele 
//
//////////////////////////////////////////////////////////////////////////////////////////
#include	<p18f1320.h>
#include	<stdio.h>
#include	<reset.h>
#include	<string.h>
#include	"AutoDcReg.h"
#include	"Prototype.h"
#include	"Battery.h"
#include	"Io.h"
#include	"Global.h"

// EEPROM ADR define
#define EEP_START_ADR 0

#pragma udata RAM_TOP

//Battery Calibration Define
typedef struct {
	int	CalVc;
	int	CalVs;
	int	CalLcv;
	int	CalLsv;
} T_BatCalV;
T_BatCalV	m_Cal;
///////////////////////////////////////////////
//	Battery ThreshHold Voltage Tabele
///////////////////////////////////////////////
far rom const T_BAT_TH_V TblBatThV = {
	BAT_Vbd,BAT_VbdTh,BAT_Vc,BAT_V100,
	BAT_V75,BAT_V50,BAT_V25,BAT_LIPPLE_V,BAT_Vl
};
///////////////////////////////////////////////
//	Calibration exec
///////////////////////////////////////////////
void calExec(T_BAT_TH_V *pBatThV)
{ 
int 	bat_v,lipple_v;
char    eep_adr;
char	led=LED_OFF;
	if (PORT_DSW4_CALIB == DSW4_CALIB){
		bat_v = AdToV(adGetBattery());
		lipple_v = AdToV5(adGetLipple());
		EepromBlockRd(EEP_START_ADR,(unsigned char*)&m_Cal,sizeof(m_Cal));	
		if (PORT_DSW1_CALIB_SEL == DSW1_ON_CHARGE){
			// Charge Voltage Set
			uartPutStrValue((T_ROM) "Wr Charge Voltage",NON_VALUE);
			m_Cal.CalVc = bat_v;
			m_Cal.CalLcv = lipple_v;
		}else{
			// Stanby Voltage Set
			uartPutStrValue((T_ROM) "Wr Stanby Voltage",NON_VALUE);
			m_Cal.CalVs = bat_v;
			m_Cal.CalLsv = lipple_v;
		}
		EepromBlockWr(EEP_START_ADR,(unsigned char*)&m_Cal,sizeof(m_Cal));	

		PORT_MLED	= MLED_ON;
		PORT_LLED 	= LLED_ON;
		ioFlash();
		while(1){
			Sleep();
		}
	}
}
///////////////////////////////////////////////
//	Calibration Data Initialize
//	<in> pBatThV:Caribration Data Buffer
///////////////////////////////////////////////
void calDataRead(T_BAT_TH_V *pBatThV)
{
		memcpypgm2ram((unsigned char*)pBatThV,&TblBatThV,sizeof(T_BAT_TH_V));
		EepromBlockRd(EEP_START_ADR,(unsigned char*)&m_Cal,sizeof(m_Cal));	
		if (PORT_DSW1_DEFAULT != DSW1_DEFAULT_VAL){
			// Use Calibration Data
			pBatThV->Vc = (m_Cal.CalVc - m_Cal.CalVs)/2+m_Cal.CalVs;
			pBatThV->LvTh = (m_Cal.CalLcv - m_Cal.CalLsv)/2+m_Cal.CalLsv;
		}
#if 1
	uartPutStrValue((T_ROM) "Vbd=",pBatThV->Vbd);
	uartPutStrValue((T_ROM) "VbdTh=",pBatThV->VbdTh);
	uartPutStrValue((T_ROM) "Vc=",pBatThV->Vc);
	uartPutStrValue((T_ROM) "V100=",pBatThV->V100);
	uartPutStrValue((T_ROM) "V75=",pBatThV->V75);
	uartPutStrValue((T_ROM) "V50=",pBatThV->V50);
	uartPutStrValue((T_ROM) "V25=",pBatThV->V25);
	uartPutStrValue((T_ROM) "LvTh=",pBatThV->LvTh);
	uartPutStrValue((T_ROM) "CalVc=",m_Cal.CalVc);
	uartPutStrValue((T_ROM) "CalLcv=",m_Cal.CalLcv);
	uartPutStrValue((T_ROM) "CalVs=",m_Cal.CalVs);
	uartPutStrValue((T_ROM) "CalLsv=",m_Cal.CalLsv);
#endif
}
